#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Measures the amount of minutes it takes for a tick/volume/range/etc. bar to form
	/// 
	/// Code Written by Tyler Moore
	/// www.FreeIndicators.Com
	/// 
	/// For modifications to this code or to have your own custom code written
	/// please visit www.MooreTechLLC.com	
    /// </summary>
    [Description("Measures the amount of minutes it takes for a tick/volume/range/etc. bar to form")]
    public class BarSpeed : Indicator
    {
        #region Variables
        // Wizard generated variables
        // User defined variables (add any user defined variables below)		
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(Color.FromKnownColor(KnownColor.Blue), PlotStyle.Bar, "TimePast"));
            Add(new Line(Color.FromKnownColor(KnownColor.Black), 0, "Zero"));
            Overlay				= false;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
            if (CurrentBar<1)
				return;
				
			DateTime eTime, pTime;							
			if (Bars.FirstBarOfSession)				
				Bars.Session.GetNextBeginEnd(Time[0],out pTime,out eTime);				 			
			else
				pTime = Time[1];
			
			TimeSpan span = Time[0].Subtract(pTime);			
            TimePast.Set(span.TotalMinutes);
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries TimePast
        {
            get { return Values[0]; }
        }

        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private BarSpeed[] cacheBarSpeed = null;

        private static BarSpeed checkBarSpeed = new BarSpeed();

        /// <summary>
        /// Measures the amount of minutes it takes for a tick/volume/range/etc. bar to form
        /// </summary>
        /// <returns></returns>
        public BarSpeed BarSpeed()
        {
            return BarSpeed(Input);
        }

        /// <summary>
        /// Measures the amount of minutes it takes for a tick/volume/range/etc. bar to form
        /// </summary>
        /// <returns></returns>
        public BarSpeed BarSpeed(Data.IDataSeries input)
        {
            if (cacheBarSpeed != null)
                for (int idx = 0; idx < cacheBarSpeed.Length; idx++)
                    if (cacheBarSpeed[idx].EqualsInput(input))
                        return cacheBarSpeed[idx];

            lock (checkBarSpeed)
            {
                if (cacheBarSpeed != null)
                    for (int idx = 0; idx < cacheBarSpeed.Length; idx++)
                        if (cacheBarSpeed[idx].EqualsInput(input))
                            return cacheBarSpeed[idx];

                BarSpeed indicator = new BarSpeed();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                Indicators.Add(indicator);
                indicator.SetUp();

                BarSpeed[] tmp = new BarSpeed[cacheBarSpeed == null ? 1 : cacheBarSpeed.Length + 1];
                if (cacheBarSpeed != null)
                    cacheBarSpeed.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheBarSpeed = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Measures the amount of minutes it takes for a tick/volume/range/etc. bar to form
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.BarSpeed BarSpeed()
        {
            return _indicator.BarSpeed(Input);
        }

        /// <summary>
        /// Measures the amount of minutes it takes for a tick/volume/range/etc. bar to form
        /// </summary>
        /// <returns></returns>
        public Indicator.BarSpeed BarSpeed(Data.IDataSeries input)
        {
            return _indicator.BarSpeed(input);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Measures the amount of minutes it takes for a tick/volume/range/etc. bar to form
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.BarSpeed BarSpeed()
        {
            return _indicator.BarSpeed(Input);
        }

        /// <summary>
        /// Measures the amount of minutes it takes for a tick/volume/range/etc. bar to form
        /// </summary>
        /// <returns></returns>
        public Indicator.BarSpeed BarSpeed(Data.IDataSeries input)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.BarSpeed(input);
        }
    }
}
#endregion
